# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import (
    Any,
    AsyncIterator,
    Awaitable,
    Callable,
    Iterator,
    Optional,
    Sequence,
    Tuple,
)

from google.apps.meet_v2beta.types import resource, service


class ListConferenceRecordsPager:
    """A pager for iterating through ``list_conference_records`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListConferenceRecordsResponse` object, and
    provides an ``__iter__`` method to iterate through its
    ``conference_records`` field.

    If there are more pages, the ``__iter__`` method will make additional
    ``ListConferenceRecords`` requests and continue to iterate
    through the ``conference_records`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListConferenceRecordsResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., service.ListConferenceRecordsResponse],
        request: service.ListConferenceRecordsRequest,
        response: service.ListConferenceRecordsResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiate the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListConferenceRecordsRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListConferenceRecordsResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListConferenceRecordsRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    def pages(self) -> Iterator[service.ListConferenceRecordsResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = self._method(self._request, metadata=self._metadata)
            yield self._response

    def __iter__(self) -> Iterator[resource.ConferenceRecord]:
        for page in self.pages:
            yield from page.conference_records

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)


class ListConferenceRecordsAsyncPager:
    """A pager for iterating through ``list_conference_records`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListConferenceRecordsResponse` object, and
    provides an ``__aiter__`` method to iterate through its
    ``conference_records`` field.

    If there are more pages, the ``__aiter__`` method will make additional
    ``ListConferenceRecords`` requests and continue to iterate
    through the ``conference_records`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListConferenceRecordsResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., Awaitable[service.ListConferenceRecordsResponse]],
        request: service.ListConferenceRecordsRequest,
        response: service.ListConferenceRecordsResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiates the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListConferenceRecordsRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListConferenceRecordsResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListConferenceRecordsRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    async def pages(self) -> AsyncIterator[service.ListConferenceRecordsResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = await self._method(self._request, metadata=self._metadata)
            yield self._response

    def __aiter__(self) -> AsyncIterator[resource.ConferenceRecord]:
        async def async_generator():
            async for page in self.pages:
                for response in page.conference_records:
                    yield response

        return async_generator()

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)


class ListParticipantsPager:
    """A pager for iterating through ``list_participants`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListParticipantsResponse` object, and
    provides an ``__iter__`` method to iterate through its
    ``participants`` field.

    If there are more pages, the ``__iter__`` method will make additional
    ``ListParticipants`` requests and continue to iterate
    through the ``participants`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListParticipantsResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., service.ListParticipantsResponse],
        request: service.ListParticipantsRequest,
        response: service.ListParticipantsResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiate the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListParticipantsRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListParticipantsResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListParticipantsRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    def pages(self) -> Iterator[service.ListParticipantsResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = self._method(self._request, metadata=self._metadata)
            yield self._response

    def __iter__(self) -> Iterator[resource.Participant]:
        for page in self.pages:
            yield from page.participants

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)


class ListParticipantsAsyncPager:
    """A pager for iterating through ``list_participants`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListParticipantsResponse` object, and
    provides an ``__aiter__`` method to iterate through its
    ``participants`` field.

    If there are more pages, the ``__aiter__`` method will make additional
    ``ListParticipants`` requests and continue to iterate
    through the ``participants`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListParticipantsResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., Awaitable[service.ListParticipantsResponse]],
        request: service.ListParticipantsRequest,
        response: service.ListParticipantsResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiates the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListParticipantsRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListParticipantsResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListParticipantsRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    async def pages(self) -> AsyncIterator[service.ListParticipantsResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = await self._method(self._request, metadata=self._metadata)
            yield self._response

    def __aiter__(self) -> AsyncIterator[resource.Participant]:
        async def async_generator():
            async for page in self.pages:
                for response in page.participants:
                    yield response

        return async_generator()

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)


class ListParticipantSessionsPager:
    """A pager for iterating through ``list_participant_sessions`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListParticipantSessionsResponse` object, and
    provides an ``__iter__`` method to iterate through its
    ``participant_sessions`` field.

    If there are more pages, the ``__iter__`` method will make additional
    ``ListParticipantSessions`` requests and continue to iterate
    through the ``participant_sessions`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListParticipantSessionsResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., service.ListParticipantSessionsResponse],
        request: service.ListParticipantSessionsRequest,
        response: service.ListParticipantSessionsResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiate the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListParticipantSessionsRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListParticipantSessionsResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListParticipantSessionsRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    def pages(self) -> Iterator[service.ListParticipantSessionsResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = self._method(self._request, metadata=self._metadata)
            yield self._response

    def __iter__(self) -> Iterator[resource.ParticipantSession]:
        for page in self.pages:
            yield from page.participant_sessions

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)


class ListParticipantSessionsAsyncPager:
    """A pager for iterating through ``list_participant_sessions`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListParticipantSessionsResponse` object, and
    provides an ``__aiter__`` method to iterate through its
    ``participant_sessions`` field.

    If there are more pages, the ``__aiter__`` method will make additional
    ``ListParticipantSessions`` requests and continue to iterate
    through the ``participant_sessions`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListParticipantSessionsResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., Awaitable[service.ListParticipantSessionsResponse]],
        request: service.ListParticipantSessionsRequest,
        response: service.ListParticipantSessionsResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiates the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListParticipantSessionsRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListParticipantSessionsResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListParticipantSessionsRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    async def pages(self) -> AsyncIterator[service.ListParticipantSessionsResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = await self._method(self._request, metadata=self._metadata)
            yield self._response

    def __aiter__(self) -> AsyncIterator[resource.ParticipantSession]:
        async def async_generator():
            async for page in self.pages:
                for response in page.participant_sessions:
                    yield response

        return async_generator()

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)


class ListRecordingsPager:
    """A pager for iterating through ``list_recordings`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListRecordingsResponse` object, and
    provides an ``__iter__`` method to iterate through its
    ``recordings`` field.

    If there are more pages, the ``__iter__`` method will make additional
    ``ListRecordings`` requests and continue to iterate
    through the ``recordings`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListRecordingsResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., service.ListRecordingsResponse],
        request: service.ListRecordingsRequest,
        response: service.ListRecordingsResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiate the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListRecordingsRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListRecordingsResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListRecordingsRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    def pages(self) -> Iterator[service.ListRecordingsResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = self._method(self._request, metadata=self._metadata)
            yield self._response

    def __iter__(self) -> Iterator[resource.Recording]:
        for page in self.pages:
            yield from page.recordings

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)


class ListRecordingsAsyncPager:
    """A pager for iterating through ``list_recordings`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListRecordingsResponse` object, and
    provides an ``__aiter__`` method to iterate through its
    ``recordings`` field.

    If there are more pages, the ``__aiter__`` method will make additional
    ``ListRecordings`` requests and continue to iterate
    through the ``recordings`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListRecordingsResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., Awaitable[service.ListRecordingsResponse]],
        request: service.ListRecordingsRequest,
        response: service.ListRecordingsResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiates the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListRecordingsRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListRecordingsResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListRecordingsRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    async def pages(self) -> AsyncIterator[service.ListRecordingsResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = await self._method(self._request, metadata=self._metadata)
            yield self._response

    def __aiter__(self) -> AsyncIterator[resource.Recording]:
        async def async_generator():
            async for page in self.pages:
                for response in page.recordings:
                    yield response

        return async_generator()

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)


class ListTranscriptsPager:
    """A pager for iterating through ``list_transcripts`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListTranscriptsResponse` object, and
    provides an ``__iter__`` method to iterate through its
    ``transcripts`` field.

    If there are more pages, the ``__iter__`` method will make additional
    ``ListTranscripts`` requests and continue to iterate
    through the ``transcripts`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListTranscriptsResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., service.ListTranscriptsResponse],
        request: service.ListTranscriptsRequest,
        response: service.ListTranscriptsResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiate the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListTranscriptsRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListTranscriptsResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListTranscriptsRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    def pages(self) -> Iterator[service.ListTranscriptsResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = self._method(self._request, metadata=self._metadata)
            yield self._response

    def __iter__(self) -> Iterator[resource.Transcript]:
        for page in self.pages:
            yield from page.transcripts

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)


class ListTranscriptsAsyncPager:
    """A pager for iterating through ``list_transcripts`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListTranscriptsResponse` object, and
    provides an ``__aiter__`` method to iterate through its
    ``transcripts`` field.

    If there are more pages, the ``__aiter__`` method will make additional
    ``ListTranscripts`` requests and continue to iterate
    through the ``transcripts`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListTranscriptsResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., Awaitable[service.ListTranscriptsResponse]],
        request: service.ListTranscriptsRequest,
        response: service.ListTranscriptsResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiates the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListTranscriptsRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListTranscriptsResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListTranscriptsRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    async def pages(self) -> AsyncIterator[service.ListTranscriptsResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = await self._method(self._request, metadata=self._metadata)
            yield self._response

    def __aiter__(self) -> AsyncIterator[resource.Transcript]:
        async def async_generator():
            async for page in self.pages:
                for response in page.transcripts:
                    yield response

        return async_generator()

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)


class ListTranscriptEntriesPager:
    """A pager for iterating through ``list_transcript_entries`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListTranscriptEntriesResponse` object, and
    provides an ``__iter__`` method to iterate through its
    ``transcript_entries`` field.

    If there are more pages, the ``__iter__`` method will make additional
    ``ListTranscriptEntries`` requests and continue to iterate
    through the ``transcript_entries`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListTranscriptEntriesResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., service.ListTranscriptEntriesResponse],
        request: service.ListTranscriptEntriesRequest,
        response: service.ListTranscriptEntriesResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiate the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListTranscriptEntriesRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListTranscriptEntriesResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListTranscriptEntriesRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    def pages(self) -> Iterator[service.ListTranscriptEntriesResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = self._method(self._request, metadata=self._metadata)
            yield self._response

    def __iter__(self) -> Iterator[resource.TranscriptEntry]:
        for page in self.pages:
            yield from page.transcript_entries

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)


class ListTranscriptEntriesAsyncPager:
    """A pager for iterating through ``list_transcript_entries`` requests.

    This class thinly wraps an initial
    :class:`google.apps.meet_v2beta.types.ListTranscriptEntriesResponse` object, and
    provides an ``__aiter__`` method to iterate through its
    ``transcript_entries`` field.

    If there are more pages, the ``__aiter__`` method will make additional
    ``ListTranscriptEntries`` requests and continue to iterate
    through the ``transcript_entries`` field on the
    corresponding responses.

    All the usual :class:`google.apps.meet_v2beta.types.ListTranscriptEntriesResponse`
    attributes are available on the pager. If multiple requests are made, only
    the most recent response is retained, and thus used for attribute lookup.
    """

    def __init__(
        self,
        method: Callable[..., Awaitable[service.ListTranscriptEntriesResponse]],
        request: service.ListTranscriptEntriesRequest,
        response: service.ListTranscriptEntriesResponse,
        *,
        metadata: Sequence[Tuple[str, str]] = ()
    ):
        """Instantiates the pager.

        Args:
            method (Callable): The method that was originally called, and
                which instantiated this pager.
            request (google.apps.meet_v2beta.types.ListTranscriptEntriesRequest):
                The initial request object.
            response (google.apps.meet_v2beta.types.ListTranscriptEntriesResponse):
                The initial response object.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        self._method = method
        self._request = service.ListTranscriptEntriesRequest(request)
        self._response = response
        self._metadata = metadata

    def __getattr__(self, name: str) -> Any:
        return getattr(self._response, name)

    @property
    async def pages(self) -> AsyncIterator[service.ListTranscriptEntriesResponse]:
        yield self._response
        while self._response.next_page_token:
            self._request.page_token = self._response.next_page_token
            self._response = await self._method(self._request, metadata=self._metadata)
            yield self._response

    def __aiter__(self) -> AsyncIterator[resource.TranscriptEntry]:
        async def async_generator():
            async for page in self.pages:
                for response in page.transcript_entries:
                    yield response

        return async_generator()

    def __repr__(self) -> str:
        return "{0}<{1!r}>".format(self.__class__.__name__, self._response)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import dataclasses
import json  # type: ignore
import re
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import gapic_v1, path_template, rest_helpers, rest_streaming
from google.api_core import exceptions as core_exceptions
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.auth.transport.requests import AuthorizedSession  # type: ignore
from google.protobuf import json_format
import grpc  # type: ignore
from requests import __version__ as requests_version

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object, None]  # type: ignore


from google.protobuf import empty_pb2  # type: ignore

from google.apps.meet_v2beta.types import resource, service

from .base import DEFAULT_CLIENT_INFO as BASE_DEFAULT_CLIENT_INFO
from .base import SpacesServiceTransport

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=BASE_DEFAULT_CLIENT_INFO.gapic_version,
    grpc_version=None,
    rest_version=requests_version,
)


class SpacesServiceRestInterceptor:
    """Interceptor for SpacesService.

    Interceptors are used to manipulate requests, request metadata, and responses
    in arbitrary ways.
    Example use cases include:
    * Logging
    * Verifying requests according to service or custom semantics
    * Stripping extraneous information from responses

    These use cases and more can be enabled by injecting an
    instance of a custom subclass when constructing the SpacesServiceRestTransport.

    .. code-block:: python
        class MyCustomSpacesServiceInterceptor(SpacesServiceRestInterceptor):
            def pre_create_space(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_create_space(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_end_active_conference(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def pre_get_space(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_get_space(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_update_space(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_update_space(self, response):
                logging.log(f"Received response: {response}")
                return response

        transport = SpacesServiceRestTransport(interceptor=MyCustomSpacesServiceInterceptor())
        client = SpacesServiceClient(transport=transport)


    """

    def pre_create_space(
        self, request: service.CreateSpaceRequest, metadata: Sequence[Tuple[str, str]]
    ) -> Tuple[service.CreateSpaceRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for create_space

        Override in a subclass to manipulate the request or metadata
        before they are sent to the SpacesService server.
        """
        return request, metadata

    def post_create_space(self, response: resource.Space) -> resource.Space:
        """Post-rpc interceptor for create_space

        Override in a subclass to manipulate the response
        after it is returned by the SpacesService server but before
        it is returned to user code.
        """
        return response

    def pre_end_active_conference(
        self,
        request: service.EndActiveConferenceRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[service.EndActiveConferenceRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for end_active_conference

        Override in a subclass to manipulate the request or metadata
        before they are sent to the SpacesService server.
        """
        return request, metadata

    def pre_get_space(
        self, request: service.GetSpaceRequest, metadata: Sequence[Tuple[str, str]]
    ) -> Tuple[service.GetSpaceRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for get_space

        Override in a subclass to manipulate the request or metadata
        before they are sent to the SpacesService server.
        """
        return request, metadata

    def post_get_space(self, response: resource.Space) -> resource.Space:
        """Post-rpc interceptor for get_space

        Override in a subclass to manipulate the response
        after it is returned by the SpacesService server but before
        it is returned to user code.
        """
        return response

    def pre_update_space(
        self, request: service.UpdateSpaceRequest, metadata: Sequence[Tuple[str, str]]
    ) -> Tuple[service.UpdateSpaceRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for update_space

        Override in a subclass to manipulate the request or metadata
        before they are sent to the SpacesService server.
        """
        return request, metadata

    def post_update_space(self, response: resource.Space) -> resource.Space:
        """Post-rpc interceptor for update_space

        Override in a subclass to manipulate the response
        after it is returned by the SpacesService server but before
        it is returned to user code.
        """
        return response


@dataclasses.dataclass
class SpacesServiceRestStub:
    _session: AuthorizedSession
    _host: str
    _interceptor: SpacesServiceRestInterceptor


class SpacesServiceRestTransport(SpacesServiceTransport):
    """REST backend transport for SpacesService.

    REST API for services dealing with spaces.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1

    """

    def __init__(
        self,
        *,
        host: str = "meet.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        url_scheme: str = "https",
        interceptor: Optional[SpacesServiceRestInterceptor] = None,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'meet.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.

            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Callable[[], Tuple[bytes, bytes]]): Client
                certificate to configure mutual TLS HTTP channel. It is ignored
                if ``channel`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you are developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
            url_scheme: the protocol scheme for the API endpoint.  Normally
                "https", but for testing or local servers,
                "http" can be specified.
        """
        # Run the base constructor
        # TODO(yon-mg): resolve other ctor params i.e. scopes, quota, etc.
        # TODO: When custom host (api_endpoint) is set, `scopes` must *also* be set on the
        # credentials object
        maybe_url_match = re.match("^(?P<scheme>http(?:s)?://)?(?P<host>.*)$", host)
        if maybe_url_match is None:
            raise ValueError(
                f"Unexpected hostname structure: {host}"
            )  # pragma: NO COVER

        url_match_items = maybe_url_match.groupdict()

        host = f"{url_scheme}://{host}" if not url_match_items["scheme"] else host

        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )
        self._session = AuthorizedSession(
            self._credentials, default_host=self.DEFAULT_HOST
        )
        if client_cert_source_for_mtls:
            self._session.configure_mtls_channel(client_cert_source_for_mtls)
        self._interceptor = interceptor or SpacesServiceRestInterceptor()
        self._prep_wrapped_messages(client_info)

    class _CreateSpace(SpacesServiceRestStub):
        def __hash__(self):
            return hash("CreateSpace")

        def __call__(
            self,
            request: service.CreateSpaceRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> resource.Space:
            r"""Call the create space method over HTTP.

            Args:
                request (~.service.CreateSpaceRequest):
                    The request object. Request to create a space.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.resource.Space:
                    `Developer
                Preview <https://developers.google.com/workspace/preview>`__.
                Virtual place where conferences are held. Only one
                active conference can be held in one space at any given
                time.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2beta/spaces",
                    "body": "space",
                },
            ]
            request, metadata = self._interceptor.pre_create_space(request, metadata)
            pb_request = service.CreateSpaceRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = resource.Space()
            pb_resp = resource.Space.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_create_space(resp)
            return resp

    class _EndActiveConference(SpacesServiceRestStub):
        def __hash__(self):
            return hash("EndActiveConference")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: service.EndActiveConferenceRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ):
            r"""Call the end active conference method over HTTP.

            Args:
                request (~.service.EndActiveConferenceRequest):
                    The request object. Request to end an ongoing conference
                of a space.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.
            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v2beta/{name=spaces/*}:endActiveConference",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_end_active_conference(
                request, metadata
            )
            pb_request = service.EndActiveConferenceRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

    class _GetSpace(SpacesServiceRestStub):
        def __hash__(self):
            return hash("GetSpace")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: service.GetSpaceRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> resource.Space:
            r"""Call the get space method over HTTP.

            Args:
                request (~.service.GetSpaceRequest):
                    The request object. Request to get a space.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.resource.Space:
                    `Developer
                Preview <https://developers.google.com/workspace/preview>`__.
                Virtual place where conferences are held. Only one
                active conference can be held in one space at any given
                time.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v2beta/{name=spaces/*}",
                },
            ]
            request, metadata = self._interceptor.pre_get_space(request, metadata)
            pb_request = service.GetSpaceRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = resource.Space()
            pb_resp = resource.Space.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_get_space(resp)
            return resp

    class _UpdateSpace(SpacesServiceRestStub):
        def __hash__(self):
            return hash("UpdateSpace")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: service.UpdateSpaceRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> resource.Space:
            r"""Call the update space method over HTTP.

            Args:
                request (~.service.UpdateSpaceRequest):
                    The request object. Request to update a space.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.resource.Space:
                    `Developer
                Preview <https://developers.google.com/workspace/preview>`__.
                Virtual place where conferences are held. Only one
                active conference can be held in one space at any given
                time.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "patch",
                    "uri": "/v2beta/{space.name=spaces/*}",
                    "body": "space",
                },
            ]
            request, metadata = self._interceptor.pre_update_space(request, metadata)
            pb_request = service.UpdateSpaceRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"], use_integers_for_enums=True
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = resource.Space()
            pb_resp = resource.Space.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_update_space(resp)
            return resp

    @property
    def create_space(self) -> Callable[[service.CreateSpaceRequest], resource.Space]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._CreateSpace(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def end_active_conference(
        self,
    ) -> Callable[[service.EndActiveConferenceRequest], empty_pb2.Empty]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._EndActiveConference(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def get_space(self) -> Callable[[service.GetSpaceRequest], resource.Space]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._GetSpace(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def update_space(self) -> Callable[[service.UpdateSpaceRequest], resource.Space]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._UpdateSpace(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def kind(self) -> str:
        return "rest"

    def close(self):
        self._session.close()


__all__ = ("SpacesServiceRestTransport",)
